// Sphereuber.cpp : class for rendering a solid sphere with uber-like lighting
// @Author: Fore June, 2014

#include <math.h>
#include <GL/glew.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include "Sphereuber.h"
#include <stdio.h>
#include <vector>
#include <string>

using namespace std;
   
const float Sphereuber::eyePos[] = {0, 0, 6.3, 1};
const float Sphereuber::lightPos[] = {1, 1, 2, 1};

// Create a Sphere object
Sphereuber::Sphereuber ( int  &success )
{
 string *vs, *fs;
  char *vsSource, *fsSource;

  // Read shader source code.
  readShaderFile( (char *) "sphereuber.vert",  &vsSource);
  readShaderFile( (char *) "sphereuber.frag",  &fsSource);
  vs = new string ( vsSource );
  fs = new string ( fsSource );
  success = createShader ( vs, fs );
  if ( !success ) {
    printf("infoLog:: %s\n", infoLog );
    return;
  }
  delete vs;       delete fs;
  delete vsSource; delete fsSource;

  createSphere (1.0f, N_SLICES, N_STACKS );
  int nVertices = vertices.size();
  int k = 0;
  int m = N_STACKS;
  int n = N_SLICES;

  nTriangles = 2 * n * (m - 1);    //number of triangles
  drawOrders = new short[3*nTriangles];
  for ( int j = 0; j < n; j++ )
  {
     for ( int i = 0; i < m-1; i++ ) {
       short j1 = (short)(j + 1);
       if ( j == n - 1 ) j1 = 0;   //wrap around
       short ia = (short)( j * m + i ) ;
       short ib = (short)( j * m + i + 1);
       short ic = (short) (j1 * m + i );
       short id = (short)( j1 * m + i + 1 );
       drawOrders[k++] = ia;
       drawOrders[k++] = ib;
       drawOrders[k++] = ic;
   
       drawOrders[k++] = ic;
       drawOrders[k++] = ib;
       drawOrders[k++] = id;
     }
  }

  sphereCoords = new float[3*nVertices];
  k = 0;
  int kk = 0;
  for ( int i = 0; i < nVertices; i++ ) {
    XYZ v = vertices[i];
    sphereCoords[k++] = v.x;  //triangle vertex coordinates
    sphereCoords[k++] = v.y;
    sphereCoords[k++] = v.z;
  }

}

Sphereuber::~Sphereuber()
{
  while (vertices.size() > 0 )
    vertices.pop_back();
}

void Sphereuber::createSphere ( float r, int nSlices, int nStacks )
{
     double phi,  theta;
     XYZ *p = new XYZ();
     XYZ *t = new XYZ();   //for texture coordinates
     const double PI = 3.1415926;
     const double TWOPI = 2 * PI;

     for ( int j = 0; j < nSlices; j++ ) {
       phi = j * TWOPI / (nSlices-1);   //0 to 2pi  (modified for texture)
       for ( int i = 0; i < nStacks; i++ ) {
         theta = i * PI / (nStacks-1);  //0 to pi
         p->x = r * (float) ( sin ( theta ) *  cos ( phi ));
         p->y = r * (float) ( sin ( theta ) *  sin ( phi ));
         p->z = r * (float)  cos ( theta );
         t->x = phi / TWOPI;   // s (column)
         t->y = 1 - theta / PI; // t (row)
         vertices.push_back ( *p  );
       }
     }
}

void Sphereuber::draw( float mvpMatrix[4][4] ) 
{
  // get handle to vertex shader's attribute variable vPosition
  int positionHandle= glGetAttribLocation(program,"vPosition");
  // Enable a handle to the sphere vertices
  glEnableVertexAttribArray( positionHandle );
  // Prepare the sphere coordinate data
  glVertexAttribPointer(positionHandle, COORDS_PER_VERTEX,
                           GL_FLOAT, false, 0, sphereCoords);
  
  // get handle to model-view projection matrix 
  int mvpMatrixHandle = glGetUniformLocation(program, "mvpMatrix");
  if (mvpMatrixHandle == -1)
     printf("No such uniform named %s\n", "mvpMatrix" );
  // pass model-view projection matrix to vertex shader
  glUniformMatrix4fv(mvpMatrixHandle, 1, GL_FALSE, &mvpMatrix[0][0] );

  int eyePosHandle = glGetUniformLocation(program, "eyePos");
  int lightPosHandle = glGetUniformLocation(program, "lightPos");
  int shininessHandle =  glGetUniformLocation(program, "shininess");
  glUniform4fv(eyePosHandle, 1, eyePos);
  glUniform4fv(lightPosHandle, 1, lightPos);
  glUniform1f(shininessHandle, shininess);
  //---------- uber light constants ---------------
  int aLoc = glGetUniformLocation(program, "a"); 
  int bLoc = glGetUniformLocation(program, "b"); 
  int ALoc = glGetUniformLocation(program, "A"); 
  int BLoc = glGetUniformLocation(program, "B"); 
  int dLoc = glGetUniformLocation(program, "d"); 
  glUniform1f (aLoc, 0.2f);  
  glUniform1f (bLoc, 0.3f);  
  glUniform1f (ALoc, 0.3f);  
  glUniform1f (BLoc, 0.4f);  
  glUniform1f (dLoc, 0.8f);    //roundness 
  
  int zNearLoc = glGetUniformLocation(program, "zNear"); 
  int zFarLoc = glGetUniformLocation(program, "zFar"); 
  int nearEdgeLoc = glGetUniformLocation(program, "nearEdge"); 
  int farEdgeLoc = glGetUniformLocation(program, "farEdge"); 

  glUniform1f (zNearLoc,   1.0f);  
  glUniform1f (zFarLoc,    2.0f);  
  glUniform1f (nearEdgeLoc, .4f);  
  glUniform1f (farEdgeLoc,  .8f);  

  // Draw all triangles
  for ( int i = 0; i < nTriangles; i++ ) 
    glDrawElements( GL_TRIANGLES, 3,
                          GL_UNSIGNED_SHORT, (drawOrders + i*3 )); 

  glDisableVertexAttribArray(positionHandle);
}

